@@ -14,18 +14,18 @@
 
 extern crate env_logger;
 extern crate openstack;
+extern crate waiter;
 
 use std::env;
-use openstack::Waiter;
+use waiter::Waiter;
 
 
 #[cfg(feature = "compute")]
 fn main() {
     env_logger::init();
 
-    let identity = openstack::auth::from_env()
+    let os = openstack::Cloud::from_env()
         .expect("Failed to create an identity provider from the environment");
-    let os = openstack::Cloud::new(identity);
 
     let id = env::args().nth(1).expect("Provide a server ID");
     let action = env::args().nth(2).expect("Provide an action");
