@@ -14,6 +14,8 @@
 
 //! Cloud API.
 
+use std::rc::Rc;
+
 use super::Result;
 use super::auth::{self, AuthMethod};
 #[allow(unused_imports)]
@@ -33,7 +35,7 @@ use super::session::Session;
 /// Provides high-level API for working with OpenStack clouds.
 #[derive(Debug, Clone)]
 pub struct Cloud {
-    session: Session
+    session: Rc<Session>
 }
 
 impl Cloud {
@@ -56,7 +58,7 @@ impl Cloud {
     /// [from_env](#method.from_env).
     pub fn new<Auth: AuthMethod + 'static>(auth_method: Auth) -> Cloud {
         Cloud {
-            session: Session::new(auth_method)
+            session: Rc::new(Session::new(auth_method))
         }
     }
 
@@ -72,7 +74,7 @@ impl Cloud {
     /// ```
     pub fn from_env() -> Result<Cloud> {
         Ok(Cloud {
-            session: Session::new(auth::from_env()?)
+            session: Rc::new(Session::new(auth::from_env()?))
         })
     }
 
@@ -88,21 +90,15 @@ impl Cloud {
     ///
     /// # fn main() { cloud_from_env().unwrap(); }
     /// ```
-    pub fn with_endpoint_interface<S>(self, endpoint_interface: S)
+    pub fn with_endpoint_interface<S>(mut self, endpoint_interface: S)
             -> Cloud where S: Into<String> {
-        Cloud {
-            session: self.session.with_endpoint_interface(endpoint_interface)
-        }
+        Rc::make_mut(&mut self.session).set_endpoint_interface(endpoint_interface);
+        self
     }
 
     /// Refresh this `Cloud` object (renew token, refetch service catalog, etc).
     pub fn refresh(&mut self) -> Result<()> {
-        self.session.auth_method_mut().refresh()
-    }
-
-    /// `Session` used with this `Cloud` object.
-    pub fn session(&self) -> &Session {
-        &self.session
+        Rc::make_mut(&mut self.session).auth_method_mut().refresh()
     }
 
     /// Build a query against flavor list.
@@ -111,7 +107,7 @@ impl Cloud {
     /// the query.
     #[cfg(feature = "compute")]
     pub fn find_flavors(&self) -> FlavorQuery {
-        FlavorQuery::new(&self.session)
+        FlavorQuery::new(self.session.clone())
     }
 
     /// Build a query against image list.
@@ -120,7 +116,7 @@ impl Cloud {
     /// the query.
     #[cfg(feature = "image")]
     pub fn find_images(&self) -> ImageQuery {
-        ImageQuery::new(&self.session)
+        ImageQuery::new(self.session.clone())
     }
 
     /// Build a query against key pairs list.
@@ -129,7 +125,7 @@ impl Cloud {
     /// the query.
     #[cfg(feature = "compute")]
     pub fn find_keypairs(&self) -> KeyPairQuery {
-        KeyPairQuery::new(&self.session)
+        KeyPairQuery::new(self.session.clone())
     }
 
     /// Build a query against network list.
@@ -138,7 +134,7 @@ impl Cloud {
     /// the query.
     #[cfg(feature = "network")]
     pub fn find_networks(&self) -> NetworkQuery {
-        NetworkQuery::new(&self.session)
+        NetworkQuery::new(self.session.clone())
     }
 
     /// Build a query against server list.
@@ -161,7 +157,7 @@ impl Cloud {
     /// ```
     #[cfg(feature = "compute")]
     pub fn find_servers(&self) -> ServerQuery {
-        ServerQuery::new(&self.session)
+        ServerQuery::new(self.session.clone())
     }
 
     /// Find a flavor by its name or ID.
@@ -176,7 +172,7 @@ impl Cloud {
     /// ```
     #[cfg(feature = "compute")]
     pub fn get_flavor<Id: AsRef<str>>(&self, id_or_name: Id) -> Result<Flavor> {
-        Flavor::new(&self.session, id_or_name)
+        Flavor::new(self.session.clone(), id_or_name)
     }
 
     /// Find an image by its name or ID.
@@ -191,7 +187,7 @@ impl Cloud {
     /// ```
     #[cfg(feature = "image")]
     pub fn get_image<Id: AsRef<str>>(&self, id_or_name: Id) -> Result<Image> {
-        Image::new(&self.session, id_or_name)
+        Image::new(self.session.clone(), id_or_name)
     }
 
     /// Find a key pair by its name or ID.
@@ -206,7 +202,7 @@ impl Cloud {
     /// ```
     #[cfg(feature = "compute")]
     pub fn get_keypair<Id: AsRef<str>>(&self, name: Id) -> Result<KeyPair> {
-        KeyPair::new(&self.session, name)
+        KeyPair::new(self.session.clone(), name)
     }
 
     /// Find an network by its name or ID.
@@ -221,7 +217,7 @@ impl Cloud {
     /// ```
     #[cfg(feature = "network")]
     pub fn get_network<Id: AsRef<str>>(&self, id_or_name: Id) -> Result<Network> {
-        Network::new(&self.session, id_or_name)
+        Network::new(self.session.clone(), id_or_name)
     }
 
     /// Find a server by its name or ID.
@@ -237,7 +233,7 @@ impl Cloud {
     /// ```
     #[cfg(feature = "compute")]
     pub fn get_server<Id: AsRef<str>>(&self, id_or_name: Id) -> Result<Server> {
-        Server::new(&self.session, id_or_name)
+        Server::new(self.session.clone(), id_or_name)
     }
 
     /// List all flavors.
@@ -338,7 +334,7 @@ impl Cloud {
     #[cfg(feature = "compute")]
     pub fn new_server<S, F>(&self, name: S, flavor: F) -> NewServer
             where S: Into<String>, F: Into<FlavorRef> {
-        NewServer::new(&self.session, name.into(), flavor.into())
+        NewServer::new(self.session.clone(), name.into(), flavor.into())
     }
 }
 
@@ -346,13 +342,7 @@ impl Cloud {
 impl From<Session> for Cloud {
     fn from(value: Session) -> Cloud {
         Cloud {
-            session: value
+            session: Rc::new(value)
         }
     }
 }
-
-impl From<Cloud> for Session {
-    fn from(value: Cloud) -> Session {
-        value.session
-    }
-}
