@@ -13,27 +13,17 @@
 // limitations under the License.
 
 //! Compute API implementation bits.
-//!
-//! # Examples
-//!
-//! ```rust,no_run
-//! use openstack;
-//!
-//! let auth = openstack::auth::from_env().expect("Unable to authenticate");
-//! let os = openstack::Cloud::new(auth);
-//!
-//! let server = os.get_server("8a1c355b-2e1e-440a-8aa8-f272df72bc32")
-//!     .expect("Unable to get a server");
-//! ```
 
 mod base;
 mod flavors;
+mod keypairs;
 mod protocol;
 mod servers;
 
 pub use self::base::V2 as ServiceType;
 pub use self::flavors::{Flavor, FlavorSummary, FlavorQuery};
-pub use self::protocol::{AddressType, RebootType, ServerAddress, ServerSortKey,
-                         ServerPowerState, ServerStatus};
-pub use self::servers::{Server, ServerQuery,
-                        ServerStatusWaiter, ServerSummary};
+pub use self::keypairs::{KeyPair, KeyPairQuery};
+pub use self::protocol::{AddressType, KeyPairType, RebootType, ServerAddress,
+                         ServerSortKey, ServerPowerState, ServerStatus};
+pub use self::servers::{NewServer, Server, ServerCreationWaiter, ServerNIC,
+                        ServerQuery, ServerStatusWaiter, ServerSummary};
